﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 스토리보드
        /// </summary>
        private Storyboard storyboard;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            NameScope.SetNameScope(this, new NameScope());

            #region 애니메이션 사각형을 생성한다.

            Rectangle animationRectangle = new Rectangle();

            animationRectangle.HorizontalAlignment = HorizontalAlignment.Left;
            animationRectangle.Width               = 100;
            animationRectangle.Height              = 20;
            animationRectangle.Fill                = Brushes.Blue;

            RegisterName("rectangle", animationRectangle);

            #endregion
            #region 시작 버튼을 생성한다.

            Button beginButton = new Button();

            beginButton.Width   = 50;
            beginButton.Height  = 30;
            beginButton.Content = "시작";

            #endregion
            #region 정지 버튼을 생성한다.

            Button pauseButton = new Button();

            pauseButton.Margin  = new Thickness(5, 0, 0, 0);
            pauseButton.Width   = 50;
            pauseButton.Height  = 30;
            pauseButton.Content = "정지";

            #endregion
            #region 재개 버튼을 생성한다.

            Button resumeButton = new Button();

            resumeButton.Margin  = new Thickness(5, 0, 0, 0);
            resumeButton.Width   = 50;
            resumeButton.Height  = 30;
            resumeButton.Content = "재개";

            #endregion
            #region 건너뛰기 버튼을 생성한다.

            Button skipToFillButton = new Button();

            skipToFillButton.Margin  = new Thickness(5, 0, 0, 0);
            skipToFillButton.Width   = 100;
            skipToFillButton.Height  = 30;
            skipToFillButton.Content = "건너뛰기";

            #endregion
            #region 3배속 버튼을 생성한다.

            Button tripleSpeedButton = new Button();

            tripleSpeedButton.Margin  = new Thickness(5, 0, 0, 0);
            tripleSpeedButton.Width   = 50;
            tripleSpeedButton.Height  = 30;
            tripleSpeedButton.Content = "3배속";

            #endregion
            #region 중단 버튼을 생성한다.

            Button stopButton = new Button();

            stopButton.Margin  = new Thickness(5, 0, 0, 0);
            stopButton.Width   = 50;
            stopButton.Height  = 30;
            stopButton.Content = "중단";

            #endregion
            #region 컨트롤 스택 패널을 생성한다.

            StackPanel controlStackPanel = new StackPanel();

            controlStackPanel.HorizontalAlignment = HorizontalAlignment.Left;
            controlStackPanel.Margin              = new Thickness(0, 30, 0, 0);
            controlStackPanel.Orientation         = Orientation.Horizontal;

            controlStackPanel.Children.Add(beginButton      );
            controlStackPanel.Children.Add(pauseButton      );
            controlStackPanel.Children.Add(resumeButton     );
            controlStackPanel.Children.Add(skipToFillButton );
            controlStackPanel.Children.Add(tripleSpeedButton);
            controlStackPanel.Children.Add(stopButton       );

            #endregion
            #region 메인 스택 패널을 생성한다.

            StackPanel mainStackPanel = new StackPanel();

            mainStackPanel.HorizontalAlignment = HorizontalAlignment.Center;
            mainStackPanel.VerticalAlignment   = VerticalAlignment.Center;
            mainStackPanel.Width               = 500;

            mainStackPanel.Children.Add(animationRectangle);
            mainStackPanel.Children.Add(controlStackPanel);

            #endregion

            #region 너비 애니메이션을 생성한다.

            DoubleAnimation widthAnimation = new DoubleAnimation(100, 500, new Duration(TimeSpan.FromSeconds(5)));

            Storyboard.SetTargetName(widthAnimation, "rectangle");
            Storyboard.SetTargetProperty(widthAnimation, new PropertyPath(Rectangle.WidthProperty));

            #endregion
            #region 스토리보드를 생성한다.

            this.storyboard = new Storyboard();

            this.storyboard.Children.Add(widthAnimation);

            #endregion

            beginButton.Click       += beginButton_Click;
            pauseButton.Click       += pauseButton_Click;
            resumeButton.Click      += resumeButton_Click;
            skipToFillButton.Click  += skipToFillButton_Click;
            tripleSpeedButton.Click += setSpeedRatioButton_Click;
            stopButton.Click        += stopButton_Click;

            Content = mainStackPanel;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 시작 버튼 클릭시 처리하기 - beginButton_Click(sender, e)

        /// <summary>
        /// 시작 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void beginButton_Click(object sender, RoutedEventArgs e)
        {
            this.storyboard.Begin(this, true);
        }

        #endregion
        #region 정지 버튼 클릭시 처리하기 - pauseButton_Click(sender, e)

        /// <summary>
        /// 정지 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void pauseButton_Click(object sender, RoutedEventArgs e)
        {
             this.storyboard.Pause(this);
        }

        #endregion
        #region 재개 버튼 클릭시 처리하기 - resumeButton_Click(sender, e)

        /// <summary>
        /// 재개 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void resumeButton_Click(object sender, RoutedEventArgs e)
        {
             this.storyboard.Resume(this);
        }

        #endregion
        #region 건너뛰기 버튼 클릭시 처리하기 - skipToFillButton_Click(sender, e)

        /// <summary>
        /// 건너뛰기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void skipToFillButton_Click(object sender, RoutedEventArgs e)
        {
             this.storyboard.SkipToFill(this);
        }

        #endregion
        #region 3배속 버튼 클릭시 처리하기 - setSpeedRatioButton_Click(sender, e)

        /// <summary>
        /// 3배속 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void setSpeedRatioButton_Click(object sender, RoutedEventArgs e)
        {
            this.storyboard.SetSpeedRatio(this, 3);
        }

        #endregion
        #region 중단 버튼 클릭시 처리하기 - stopButton_Click(sender, e)

        /// <summary>
        /// 중단 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void stopButton_Click(object sender, RoutedEventArgs e)
        {
             this.storyboard.Stop(this);
        }

        #endregion
    }
}